#!/usr/bin/perl -w
#
# Author:  mpryor@sprintmail.com
# script:	tlcom.pl	
# Description: opens the Definition files for TLViewer and builds the Mysql DB: tlcom
# keywords: mysql query database xcom registry
# Date:  10/3/01 4:05PM
# UrlRef: http://home.sprintmail.com/promo.htm
# RelatedDoc=tlcom.sql
# 
# ------------------------------------------------
#
# --- revision history ---------------------------

#  works for MySQL or Access - change the config file  
# --- end of revisions ---------------------------

# ----------------------------------------------------------------------------

########################################################################

###################
# preparing stuff #
###################
#use strict;
my $tStart = Win32::GetTickCount;

# kinda a rc-file...
my $opts = require '/etc/mysql-tlcom.config';
#my $opts = require '/etc/access-tlcom.config';

# perl-stuff to load
#

use DBI;
#use File::Find;
use File::stat;
# module from CPAN
use Text::ParseWords;

#declare vars
my (%params);  # holds the data file names and the field names

$| = 1;
# opening necessary files for logging 
open(LOG, ">>$$opts{'log_file'}") 
	or die "can't open $$opts{'log_file'} in append-mode: $!\n";
open(ERRLOG, ">>$$opts{'log_error'}")
        or die "can't open $$opts{'log_error'} in append-mode: $!\n";

	my $HANDLE = "LOG";
   my $ERRHANDLE = "ERRLOG";

##########################
# everything is prepared #
# main-routine starts    #
##########################

# tell the user who we are
print STDOUT "\n tlcom\nVersion 0.03\n\n";

print $ERRHANDLE `Make_tlcom.bat`, "\n" ;
#or die "failed to build the tables: $! \n"; # rebuild the tables
 
# connect to the mysql-deamon
my $dbh =eval { connect_mysql();};

# @fnames = sort { @{$a}[0] cmp @{$b}[0] } @fnames;

# recreate the tables

# open the dbTypes.dat file for reading
# read each line, one by one, into a hash.
# hash : %types,  keys: guid, vers, help, path, libname

# defs is the path to data file
# fields is an anon array of the table field names

%params = ( defs => 'dbTypes.dat',
            tblname => 'typelib', 
            fields => [  'typelib', 'version', 'help_string', 'pathid', 'libname'],
            lookup => [3]
            ); 

populate_table( $dbh,\%params ); 

%params = ( defs => 'dbPID.dat',
            tblname => 'progid',  
            fields => [  'progid', 'clsidid', 'help_string', 'pathid', 'typelibid'],
            lookup => [1,3,4]
            ); 

populate_table( $dbh, \%params );       

      
#disconnecting from mysql-deamon
print STDOUT "\ncleaning up...";
$dbh->disconnect(); 
print 'Elapsed= '. (Win32::GetTickCount() - $tStart) / 1000 . " sec\. \n";
print STDOUT "\ndisconnected from mysql-deamon. Finished! Bye!\n";
# guess we're done

#####################################
# now only sub-routines are defined #
#####################################

sub populate_table {
    my $dbh = shift;
    my $href = shift;
    #print " $href->{'defs'} \n";
    my $data = $href->{defs}; # filename
    my @lkups = @{$href->{lookup}}; # anon array of id fields
    my $tblname = $href->{tblname};
    my @fields = @{$href->{fields}};
    my (@rec,$id, $table);
    
    my @olrec = ( 'libname.class', 0, 'something about this object', 0, undef ); 
    # prepare the insert placeholders for this table
    my $sth = $dbh->prepare(qq{
    INSERT INTO $tblname ($fields[0],$fields[1],$fields[2],$fields[3],$fields[4]) VALUES (?, ?, ?, ?,?)
    }) || die $dbh->errstr;

    open( IN, "<$data") or die "failed to open $data \n";
    $_ = <IN>; # discard the first line
    print "\n $_ \n";
    my ($ltot, $flin, $sback) = '';
    
    $sback = "\b" x 23;
    while  (<IN>) {    
    chomp;
    $ltot = $.+ $-;
    
    #s/(\\)/$1$1/g;
    #print " $_ \n";
    #s/'/\\'/ig;

    # parsewords will mangle a record w/ single quote - escape it
    s/('|`|&)/\\$1/ig; 
    
    @rec = &parse_line(',', 1, $_);

# repeated records will have integer pointers -1, -2, etc - test for this    
    if ( $rec[1] !~ m/^\s?-\d+/ ) {

# show progress without scrolling - uses backspaces for position control
    $ltot = substr( $rec[0],0, 10);
    $flin = sprintf "% 5d", $.;
    print $sback;
    print "$ltot  line $flin";
    #last if ( $.== 5);
    
    for my $j ( 0 .. $#rec) {
    # remove leading blanks 
        $rec[$j] =~ s/^\s//;
    }

    # field 4 is always a path, escape the back slashes
    $rec[3] =~s/(\\)/$1$1/g; 

    # unescape the singe quote
    $rec[2] =~s/\\'/'/g; 

    # do the lookups
    foreach my $i ( @lkups ) {
        $id = $fields[$i];
        $table = $id;
        $table =~ s/id$//i;
        $id =~ s/id$/\.id/i;
        #print "id= $id table= $table\n";
        if ( $rec[$i] =~ m/^\s?\*|^\s?not/i ) {
        # skip the lookup for a null typelib or path       
            undef $rec[$i]  ;            
        } 
        next, unless (defined $rec[$i]);   
        $rec[$i] = LookUp_ID( $id , $table  , $table , $rec[$i] );    
    } # end lookups
    }
    else #set the data to the olrec
    {
    # 0 field is version dependent Progid
    for my $k (1 .. $#olrec ) {
        $rec[$k] = $olrec[$k];
    }
    }
    
# only reset the olrec if ther is No pointer in the second field
    if ( $rec[1] !~ m/^\s?-/ ) {
    @olrec = @rec;
    #print "@olrec \n";
    }
    eval{  
    $sth->execute( $rec[0],$rec[1],"$rec[2]",$rec[3],$rec[4]  );
    };
    if ( $DBI::errstr ){
    print $ERRHANDLE "$DBI::errstr \n";
    }
    #last;
    }
    close IN;

} # end of populate_table

sub binary_search {
# msp 8-10-01
#
    my ($array, $word) = @_;
	if ( (scalar @$array ) == 0 ) { return -1};
	my ($min, $max) = ( 0, @$array -1 );
	use integer;
	my $try = ($min+$max)/2; 
    while ( $min < $max ) { 
		#(@{$array->[$try]}[2] . '/' . @{$array->[$try]}[0]  
		my $cmp = $word cmp @{$array->[$try]}[0];	
		if( $cmp == 0 ) {		
		return $try;
		last;
			
		} elsif ( $cmp < 0 ) {	
			$max = $try;
			$try = ($try + $min) / 2;	
		} else {
			$min = $try+1;
			$try = ($try + $max) / 2;	
		}	
	}
	# bug - check for failure on the last element of fnames
my	$cmp = $word cmp (@{$array->[@$array-1]}[0]);
	if ($cmp == 0 ) {return  @$array-1;}	
    return -1;              # The word isn't there.
} # end binary_search()


# returns $dbh 
sub connect_mysql {
my $dbh ;
my $dsn = "DBI:$$opts{'driver'}:database=$$opts{'database'};host=$$opts{'host'};port=$$opts{'port'}";
if ( $$opts{'driver'} =~ /odbc/i ) {
    $dsn = "DBI:$$opts{'driver'}(RaiseError=>1):$$opts{'database'}";
    #print " $dsn \n";
    $dbh = DBI->connect($dsn,undef,undef) or die "cant connect to user dsn\n";;
}
else
{
 $dbh = DBI->connect($dsn,$$opts{'user'},$$opts{'password'}) or die "cant connect to $$opts{'host'}\n";
}
return $dbh;
}

sub LookUp_ID {
	my ($ind, $tbl,$col, $val ) = @_;
    my ($rth, $query2);
    my $query = qq(SELECT $ind from $tbl WHERE $col='$val');                      
	my $sth = $dbh->prepare($query);
   	$sth->execute() or die("Unable to execute $query", $dbh->errstr);      
   	my $retid = $sth->fetchrow_array;
    
	$sth->finish();
	if (  ! $retid ) {
	# no entry found for this artist, insert the val and get the id
    
	$query = qq(INSERT into $tbl ($col) values ('$val') ); 
    $sth = $dbh->prepare($query);
   	$sth->execute() or db_err("Unable to execute $query", $dbh->errstr);
    if ( $$opts{'driver'} =~ /odbc/i ) { 
    $query2 = qq(  SELECT Max(id) from $tbl );
    $rth =  $dbh->prepare($query2);    
        # awkward method to get the Last_INSERTID() for ODBC
    $rth->execute() or db_err("Unable to execute $query2", $dbh->errstr);   
    $retid = $rth->fetchrow_arrayref->[0]; 
    #print "id= $retid \n";
    $rth->finish();
    }
    else      
	{$retid = $sth->{mysql_insertid}; }
	$sth->finish();
}
return $retid;
}

###############################################

__END__

